<?php

/**
Retrieves the user credits from Content Raccoon API.
@return int|false The number of user credits or false if the API response is invalid.
*/

function textwincoil_get_credits() {
	
    $user_token = get_option('textwincoil_user_token');
    $website_token = get_option('textwincoil_website_token');

    if ($website_token && $user_token) {
        $response = wp_remote_post('https://api.textwin.co.il/get_user_credits', array(
            'method' => 'POST',
            'timeout' => 15,
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'headers' => array('Cache-Control' => 'no-cache'),
            'body' => array(
                'website_token' => $website_token,
                'user_token' => $user_token,
                'website_url' => get_site_url(),
            ),
            'cookies' => array(),
        ));

        if (!$response || is_wp_error($response)) {
            error_log('Error: ' . print_r($response, true));
            return 0;
        }

        $body = json_decode(wp_remote_retrieve_body($response));

        if (!isset($body->user_credits)) {
            error_log('Error: nothing returned!');
            return 0;
        }

        // Update option value and return the number of credits
        update_option('textwincoil_user_credits', $body->user_credits);
        return $body->user_credits;
    } else {
        // If tokens are not set, return 0
        update_option('textwincoil_user_credits', 0);
        return 0;
    }
}

 

// Helper function to log in to the API and get a token
function textwincoil_login( $email, $user_token ) {
    $email = sanitize_email( $email );
    $user_token = sanitize_text_field( $user_token );
    $website_url = esc_url_raw( get_site_url() );

    $response = wp_remote_post( 'https://api.textwin.co.il/login', array(
        'method' => 'POST',
        'timeout' => 90,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking' => true,
        'headers' => array( 'Cache-Control' => 'no-cache' ),
        'body' => array(
            'email'    => $email,
            'user_token' => $user_token,
            'website_url' => $website_url,
        ),       
        'cookies' => array()
    ));

    if ( ! $response || is_wp_error( $response ) ) {
        echo 'Error: ' . esc_html( print_r( $response, true ) );
        delete_option( 'textwincoil_email' );
        delete_option( 'textwincoil_user_token' );
        delete_option( 'textwincoil_website_token' );
        return false;
    }
    $body = json_decode( wp_remote_retrieve_body( $response ) );

    if ( ! $body->token ) {
        echo 'Error: no token returned';
        delete_option( 'textwincoil_email' );
        delete_option( 'textwincoil_user_token' );
        delete_option( 'textwincoil_website_token' ); 
        return false;
    }

    update_option( 'textwincoil_email', $email );
    update_option( 'textwincoil_user_token', $user_token );
    update_option( 'textwincoil_website_token', $body->token ); 

    if ( isset( $body->user_credits ) ) { 
        update_option( 'textwincoil_user_credits', $body->user_credits ); 
    } 

    return $body->token;
}


// Helper function to validate the API token
function textwincoil_validate_token( $token ) {
	
    $token = sanitize_text_field( $token );
	
    $response = wp_remote_post( esc_url( 'https://api.textwin.co.il/validate' ), array(
		  'method' => 'POST',
		  'timeout' => 90,
		  'redirection' => 5,
		  'httpversion' => '1.0',
		  'blocking' => true,
		  'headers' => array( 'Cache-Control' => 'no-cache' ),
		  'body' => array(
            'token' => $token,
            'website_url' => esc_url( get_site_url() ),
			'user_token' => get_option( 'textwincoil_user_token' ),
			'email' => get_option( 'textwincoil_email' ),
		  ),
		  'cookies' => array()
		)
	);
	
    if ( ! $response || is_wp_error( $response ) ) {
        delete_option( 'textwincoil_website_token' );
        delete_option( 'textwincoil_user_token' );
        delete_option( 'textwincoil_email' );
        return false;
    } 
	
	
    $body = json_decode( wp_remote_retrieve_body( $response ) );
	//print_r($body);
	
    return (isset($body->valid)) ? (bool) $body->valid : false;
}


function textwincoil_check_post_progress($post_id, $post_token) {
	$website_token = get_option('textwincoil_website_token');
    
	$data = array(
		'post_id' => absint($post_id),
		'post_token' => sanitize_text_field($post_token),
		'website_token' => sanitize_text_field($website_token),
		'website_url' => esc_url(get_site_url()),
	);

	$response = wp_remote_post('https://api.textwin.co.il/content_progress', array(
		'method' => 'POST',
		'timeout' => 45,
		'redirection' => 5,
		'httpversion' => '1.0',
		'blocking' => true,
		'headers' => array(),
		'body' => $data,
		'cookies' => array()
	));

	if (is_wp_error($response)) {
		return false;
	}
 
	$response_body = json_decode($response['body'], true);

	if (isset($response_body['post_progress']) && is_numeric($response_body['post_progress'])) {
		$data = array("post_progress" => absint($response_body["post_progress"]), "post_progress_label" => sanitize_text_field($response_body["post_progress_label"]));

		if ($response_body['post_progress'] == 100 && !empty($response_body['post_content']) && !empty($response_body['post_title'])) {
			update_post_meta($post_id, 'textwincoil_post_content', wp_kses_post($response_body['post_content']));
			update_post_meta($post_id, 'textwincoil_post_title', sanitize_text_field($response_body['post_title']));
			$data['post_title'] = sanitize_text_field($response_body['post_title']);
			$data['post_content'] = wp_kses_post($response_body['post_content']);
		}

		return $data;
	}

	return false;
}


add_action( 'wp_ajax_textwincoil_check_post_progress', 'textwincoil_check_post_progress_callback' );
function textwincoil_check_post_progress_callback() {
	
    $post_id = isset( $_POST['post_id'] ) ? intval( $_POST['post_id'] ) : 0;
    $post_token = isset( $_POST['post_token'] ) ? sanitize_text_field( $_POST['post_token'] ) : '';

	// Validate the input
	if ( empty( $post_id ) || empty( $post_token ) ) {
		wp_send_json_error( array( 'error' => 'Invalid input data' ) );
		return;
	}

	// Perform the operation
    $result = textwincoil_check_post_progress( $post_id, $post_token );
	
	// Send the response
    if ( $result ) {
        wp_send_json_success( $result );
    } else {
        wp_send_json_error( array( 'error' => 'Post content update failed' ) );
    }
}



add_action('wp_ajax_textwincoil_creator', 'textwincoil_creator');
function textwincoil_creator() {
	
	// Check if the form data was submitted
	if ( isset( $_POST['textwincoil_subject'] ) ) {
		
		// Sanitize and validate the form data
		$title = isset( $_POST['textwincoil_post_title'] ) ? sanitize_text_field( $_POST['textwincoil_post_title'] ) : '';
		$post_subject = sanitize_text_field( $_POST['textwincoil_subject'] );
		$section_count = isset( $_POST['textwincoil_section_count'] ) ? absint( $_POST['textwincoil_section_count'] ) : 0;
		$section_headings = isset( $_POST['textwincoil_section_headings'] ) ? array_map( 'sanitize_text_field', $_POST['textwincoil_section_headings'] ) : array();
		$length = isset( $_POST['textwincoil_length'] ) ? absint( $_POST['textwincoil_length'] ) : 0;
		$format_type = isset( $_POST['textwincoil_format_type'] ) ? sanitize_text_field( $_POST['textwincoil_format_type'] ) : '';
		
		$audience = isset( $_POST['textwincoil_audience'] ) ? sanitize_text_field( $_POST['textwincoil_audience'] ) : '';
		$purpose = isset( $_POST['textwincoil_purpose'] ) ? sanitize_text_field( $_POST['textwincoil_purpose'] ) : '';
		$tone = isset( $_POST['textwincoil_tone'] ) ? sanitize_text_field( $_POST['textwincoil_tone'] ) : '';
		$keywords = isset( $_POST['textwincoil_keywords'] ) ? array_map( 'sanitize_text_field', $_POST['textwincoil_keywords'] ) : array();
		$manual_keywords = isset( $_POST['textwincoil_manual_keywords'] ) ? sanitize_text_field( $_POST['textwincoil_manual_keywords'] ) : '';
		
		$featured_image = isset( $_POST['textwincoil_image'] ) ? true : false;
		$faqs = isset( $_POST['textwincoil_faqs'] ) ? true : false;
		$informative_table = isset( $_POST['textwincoil_table'] ) ? true : false;
		$relevant_video = isset( $_POST['textwincoil_video'] ) ? true : false;
		$add_intro = isset( $_POST['textwincoil_intro'] ) ? true : false;
		
		$step_number = isset( $_POST['textwincoil_step_number'] ) ? absint( $_POST['textwincoil_step_number'] ) : 0;
		$post_token = isset( $_POST['textwincoil_post_token'] ) ? sanitize_text_field( $_POST['textwincoil_post_token'] ) : '';
		
		$email = get_option( 'textwincoil_email' );
		$user_token = get_option( 'textwincoil_user_token' );
		
		// Prepare the form data for sending to contentraccoon
		$form_data = array(
			'website_token' => get_option( 'textwincoil_website_token' ),
			'user_token' => $user_token,
			'website_url' => esc_url( get_site_url() ),
			'title' => $title,
			'post_subject' => $post_subject,
			'section_count' => $section_count,
			'section_headings' => $section_headings,
			'length' => $length,
			'format_type' => $format_type,
			'audience' => $audience,
			'purpose' => $purpose,
			'tone' => $tone,
			'keywords' => $keywords,
			'manual_keywords' => $manual_keywords,
			'featured_image' => $featured_image,
			'faqs' => $faqs,
			'informative_table' => $informative_table,
			'relevant_video' => $relevant_video,
			'add_intro' => $add_intro,
			'step_number' => $step_number,
			'post_token' => $post_token, 
		);

		// Send the form data to contentraccoon for processing
		$response = wp_remote_post( 'https://api.textwin.co.il/creator', array(
			'method' => 'POST',
			'timeout' => 130,
			'redirection' => 5,
			'httpversion' => '1.0',
			'blocking' => true,
			'headers' => array( 'Cache-Control' => 'no-cache' ),
			'body' => $form_data,
			'cookies' => array()
		));
		 
		// Check if the request was successful
		if ( is_wp_error( $response ) ) {
			// Return an error message
			wp_send_json_error( array( 'error' => $response->get_error_message() ) );
			
		} else {
			// Get the response data
			$data = json_decode( wp_remote_retrieve_body( $response ), true ); 

			// Check if the response was successful
			if ( isset( $data['success'] ) && $data['success'] ) {
				// Return the post content
				wp_send_json( $data );
			} else {
				// Return an error message 
				wp_send_json( array( 'bd' => $response, 'errors' => $data['errors'] ) );
			}
		}
	}

	// Return an error message if no form data was submitted
	wp_send_json_error( array( 'error' => 'No form data submitted' ) );

}

